/*******************************************************************************
 *
 * FileName:	sbmp.c
 *
 *	This file includes the functions for saving a image into
 *	a BMP (MS-Windows bitmap format) file.
 *
 * Copyright (c) 2002-2003.  Sensoray Co., Inc
 * Author:                   Charlie X. Liu
 *
 * All rights reserved.
 *
 *-----------------------------------------------------------------------------*/
//*
//* Redistribution and use in source and binary forms, with or without
//* modification, are permitted provided that the following conditions
//* are met:
//*
//* 1. Redistributions of source code must retain the above copyright
//*    notice, this list of conditions and the following disclaimer.
//* 2. Redistributions in binary form must reproduce the above copyright
//*    notice, this list of conditions and the following disclaimer in the
//*    documentation and/or other materials provided with the distribution.
//* 3. All advertising materials mentioning features or use of this software
//*    must display the following acknowledgement:
//*      This product includes software developed by Charlie X. Liu
//* 4. The name of the author may not be used to endorse or promote products
//*    derived from this software without specific prior written permission.
//*
//* THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
//* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//* DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
//* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
//* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
//* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
//* ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//* POSSIBILITY OF SUCH DAMAGE.
//*
//*******************************************************************************/

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include "615.h"
//#include <stdlib.h>


/**********************************************************
 * Open a BMP file for image to save to
 **********************************************************/
int OpnFileBMP (char *filename, int *pfh)
{

	if ((*pfh = open(filename, O_WRONLY | O_CREAT, 0666)) < 0) {
		printf("BMP file open failed\n");
		return -1;
	}
	return 0;

}

/**********************************************************
 * Make BMP header and save it to the BMP file
 **********************************************************/
int SvHeaderBMP (int fh, int width, int height, BOOL color)
{
    BITMAPFILEHEADER bfh;
    BITMAPINFOHEADER bih;
    int bmpsize;
    int i; long value;
    BOOL ecode = FALSE;
    /*const*/ long SIZE_BFH = 14; 			//sizeof (BITMAPFILEHEADER);	/* 14 */
    /*const*/ long SIZE_BIH = 40;			//sizeof (BITMAPINFOHEADER);	/* 40 */
    /*const*/ long SIZE_CLR = 256 * sizeof (RGBQUAD);	/* 256*4=1024 */

    // Fill in the BITMAPFILEHEADER structure
    bfh.bfType = * (WORD *) "BM";		//bmp signature
    bmpsize = width * height * (color ? 3 : 1);
    bfh.bfOffBits = SIZE_BFH + SIZE_BIH + (color ? 0 : SIZE_CLR);	// color? 54 : 1078
    //printf("BMP header lenth = %d \n", bfh.bfOffBits);
    bfh.bfSize = bmpsize + bfh.bfOffBits;
    bfh.bfReserved1 = 0;			//must be zero
    bfh.bfReserved2 = 0;			//must be zero

    // Fill in the BITMAPINFOHEADER structure
	bih.biSize = SIZE_BIH;			//must be 40, sizeof (BITMAPINFOHEADER)
	bih.biWidth = width; 
	bih.biHeight = height; 
	bih.biPlanes = 1;			//must be set to 1
	bih.biBitCount = 8 * (color ? 3 : 1);	// color? 24 : 8
	bih.biCompression = 0;			//compression (none)
	bih.biSizeImage = bmpsize;
	bih.biXPelsPerMeter = 2835;		//horizontal resolution: pixels/meter
	bih.biYPelsPerMeter = 2835;		//vertical resolution: pixels/meter
	bih.biClrUsed = (color ? 0 : 256);		
			//color:		0 - use the max number of colors corresponding
			//			    to the value of biBitCount menber
			//monochrome: 	      256 - 2^8 level, for gray-scale 
	bih.biClrImportant = (color ? 0 : 256);
			//color:		0 - all color are required
			//monochrome: 	      256 - 2^8, numbers of color indexs required for
			//			    displaying the bitmap

	// Write BITMAPFILEHEADER to file
	//if (write(fh, &bfh, SIZE_BFH)<0) {ecode = -1; return ecode;}
	write(fh, &(bfh.bfType), 2);
	write(fh, &(bfh.bfSize), 4);
	write(fh, &(bfh.bfReserved1), 2);
	write(fh, &(bfh.bfReserved2), 2);
	write(fh, &(bfh.bfOffBits), 4);

	// Write BITMAPINFOHEADER to file
	if (write(fh, &bih, SIZE_BIH)<0) {ecode = -1; return ecode;}

	// If monochrome image - write color table to file
    if (!color) {
		for (i=0; i<256; i++) 
		{
			value = i + (i<<8) + (i<<16);	//Make the RGBQUAD structure:
							//	rgbBlue		= i; 
							//	rgbGreen	= i;
							//	rgbRed		= i;
							//	rgbReserved	= 0, must be zero.
			//if (write(fh, &value, 4)<0) {ecode = -1; return ecode;};
			write(fh, &value, 4);
		}
	}	

	return ecode;

}

/**********************************************************
 * Save data to the BMP file
 **********************************************************/
int SvDataBMP (int fh, char *buffer, int length)
{
    BOOL ecode = FALSE;

	// Write bitmap data to the BMP file
	if (write(fh, buffer, length)<0) {ecode = -1; return ecode;}
	return ecode;

}

/**********************************************************
 * Close the BMP file
 **********************************************************/
int ClsFileBMP (int fh)
{

  if (close(fh)<0) return -1;
  return 0;

}














